'' =================================================================================================
''
''   File....... jm_ds3231.spin
''   Purpose.... DS3231 RTC interface (uses 24-hr format)
''   Author..... Jon "JonnyMac" McPhalen  
''               Copyright (C) 2014-2016 Jon McPhalen    
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 07 DEC 2016
''
'' =================================================================================================


con { fixed io pins }

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30

  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con

  DS3231_WR = %1101_000_0                                       ' i2c slave id
  DS3231_RD = %1101_000_1  
                                                                 
  #$00, R_SCS, R_MNS, R_HRS                                     ' time registers
  #$03, R_DAY, R_DATE, R_MON, R_YR                              ' date registers        

  #$07, R_A1SCS, R_A1MNS, R_A1HRS                               ' alarm 1 time registers 
  #$0A, R_A1DYDT                                                ' alarm 1 day / date

  #$0B, R_A2MNS, R_A2HRS                                        ' alarm 2 time registers
  #$0D, R_A2DYDT                                                ' alarm 2 day / date
  
  #$0E, R_CTRL                                                  ' control register
  #$0F, R_STATUS                                                ' status register

  #$10, R_OFFSET                                                ' aging offset

  #$11, R_TMSB, R_TLSB                                          ' temp registers                
  

obj

  i2c : "jm_i2c"


pub null

  ' This is not a top-level object
  

pub start(sclpin, sdapin) | check

'' Connects DS3231 to user-defined I2C buss

  i2c.setupx(sclpin, sdapin)
  if (present)
    check := rd_reg(R_STATUS)
    if (check & $80)                                            ' clear OSF bit if set
      check &= $7F
      wr_reg(R_STATUS, check)


pub present

  return i2c.present(DS3231_WR)   

  
pub set_time(sc, mn, hr)

'' Set seconds, minutes, and hours registers
'' -- values are BCD

  i2c.start
  i2c.write(DS3231_WR)
  i2c.write(R_SCS)
  i2c.write(sc)
  i2c.write(mn)
  i2c.write(hr)
  i2c.stop


pub set_calendar(dy, dt, mo, yr)

'' Set calendar
'' -- values are BCD

  i2c.start
  i2c.write(DS3231_WR)
  i2c.write(R_DAY)
  i2c.write(dy)
  i2c.write(dt)
  i2c.write(mo)
  i2c.write(yr)
  i2c.stop


pub write(n, reg, p_src)

'' Write n BCD registers at p_src to DS3231
'' -- values at p_src are BCD

  i2c.start
  i2c.write(DS3231_WR)
  i2c.write(reg)
  repeat n
    i2c.write(byte[p_src++])  
  i2c.stop


pub wr_reg(reg, value)

'' Writes specific BCD register in DS3231

  write(1, reg, @value)


pub wr_time(p_src)

'' Write time (secs, mins, hours) registers at p_src to DS3231

  write(3, R_SCS, p_src)


pub wr_calendar(p_src)

'' Write calendar (day, date, month, year) registers at p_src to DS3231

  write(4, R_DAY, p_src)


pub read(n, p_dest, reg)

'' Reads n regsiters at addr in DS3231 to p_dest

  i2c.start
  i2c.write(DS3231_WR)
  i2c.write(reg)                                                ' set address pointer
  i2c.start                                                     ' restart for read
  i2c.write(DS3231_RD)
  repeat (n-1)
    byte[p_dest++] := i2c.read(i2c#ACK)
  byte[p_dest++] := i2c.read(i2c#NAK)
  i2c.stop


pub rd_reg(reg) | value

'' Reads single registers from DS3231

  read(1, @value, reg)

  return value.byte[0]
  

pub rd_time(p_dest)

'' Reads clock registers (secs, mins, hours) from DS3231 to p_dest

  read(3, p_dest, R_SCS)


pub rd_calendar(p_dest)

'' Reads calendar registers (day, date, month, year) from DS3231 to p_dest

  read(4, p_dest, R_DAY)


pub time_code

'' Returns current time to formatted long: $00_hh_mm_ss

  read(3, @result, R_SCS)


pub date_code 

'' Returns current time to formatted long: $00_yr_mo_dt

  read(3, @result, R_DATE)


con

  { ----------------------------------- }
  {  F O R M A T T E D   S T R I N G S  }
  { ----------------------------------- }

  
pub time_hhmmss(p_src) | work 

'' Returns pointer to formatted time string
'' -- formatted string of registers at p_src
'' -- user -1 for internal RTC registers

  if (p_src < 0)
    read(3, @work, R_SCS)                                       ' internal clock 
  else
    bytemove(@work, p_src, 3)                                   ' external values

  StrLong[0] := work.byte[2] >>   4 + "0"                       ' convert to ASCII
  StrLong[1] := work.byte[2] &  $0F + "0" 
  StrLong[2] := ":"                        
  StrLong[3] := work.byte[1] >>   4 + "0" 
  StrLong[4] := work.byte[1] &  $0F + "0"
  StrLong[5] := ":"
  StrLong[6] := work.byte[0] >>   4 + "0" 
  StrLong[7] := work.byte[0] &  $0F + "0" 

  return @StrLong


pub time_ampm(p_src) | work, hr

'' Returns pointer to formatted time string
'' -- formatted string of registers at p_src
'' -- user -1 for internal RTC registers

  if (p_src < 0)
    read(3, @work, R_SCS)                                       ' internal clock 
  else
    bytemove(@work, p_src, 3)                                   ' external values

  hr := bcd2dec(work.byte[2])                                   ' convert to decimal

  case hr                                                       ' adjust for am/pm
    0..11 :
      TimeStr[9] := "A"
      if (hr == 0)
        hr := 12

    12..23 :
      TimeStr[9] := "P"
      if (hr > 12)
        hr -= 12

  TimeStr[0] := hr  / 10 + "0"                                  ' convert to ASCII
  TimeStr[1] := hr // 10 + "0" 
  TimeStr[2] := ":"                        
  TimeStr[3] := work.byte[1] >>   4 + "0" 
  TimeStr[4] := work.byte[1] &  $0F + "0"
  TimeStr[5] := ":"
  TimeStr[6] := work.byte[0] >>   4 + "0" 
  TimeStr[7] := work.byte[0] &  $0F + "0" 

  return @TimeStr


pub date_mmddyy(p_src) | work

'' Returns pointer to formatted date
'' -- formatted string of registers at p_src
'' -- user -1 for internal RTC registers

  if (p_src < 0)
    read(3, @work, R_DATE)                                      ' internal clock
  else
    bytemove(@work, p_src, 3)                                   ' external values

  StrLong[0] := work.byte[1] >>   4 + "0"  
  StrLong[1] := work.byte[1] &  $0F + "0"  
  StrLong[2] := "/"                       
  StrLong[3] := work.byte[0] >>   4 + "0" 
  StrLong[4] := work.byte[0] &  $0F + "0" 
  StrLong[5] := "/"                       
  StrLong[6] := work.byte[2] >>   4 + "0" 
  StrLong[7] := work.byte[2] &  $0F + "0" 

  return @StrLong


pub today

'' Return RTC day of week, 1..7

  return rd_reg(R_DAY)
  

pub day_short(d)

'' Return pointer to string with name of day, e.g. "MON"
'' -- use 1..7 to print specific day name 
'' -- use -1 for current RTC day

  if (d < 0)
    d := today - 1                                              ' z-align for strings
  else
    d -= 1
    if ((d < 0) or (d > 6))
      return @Unknown

  return @NameShort + (d << 2)                                  ' 4 chars ea


pub day_long(d) | p_str

'' Return pointer to string with name of day, e.g. "Monday"
'' -- use 1..7 to print specific day name
'' -- use -1 for current RTC day

  if (d < 0)
    d := today - 1                                              ' z-align for strings
  else
    d -= 1
    if ((d < 0) or (d > 6))
      return @Unknown   

  p_str := @NameLong                                            ' point to start of names

  repeat d
    p_str += strsize(p_str) + 1                                 ' skip over to target string

  return p_str
  
      
con

  { ------------------------- }
  {   C O N V E R S I O N S   }
  { ------------------------- }


pub bcd2dec(bcd)

  return ((bcd >> 4) * 10) + (bcd & $0F)


pub dec2bcd(dec)

  return ((dec / 10) << 4) | (dec // 10)


dat

  Unknown               byte    "???", 0

  StrShort              byte    "00:00", 0
  StrLong               byte    "00:00:00", 0              

  TimeStr               byte    "hh:mm:ss xM", 0

  NameShort             byte    "SUN", 0
                        byte    "MON", 0
                        byte    "TUE", 0
                        byte    "WED", 0
                        byte    "THU", 0
                        byte    "FRI", 0
                        byte    "SAT", 0

  NameLong              byte    "Sunday", 0 
                        byte    "Monday", 0 
                        byte    "Tuesday", 0 
                        byte    "Wednesday", 0
                        byte    "Thursday", 0 
                        byte    "Friday", 0 
                        byte    "Saturday", 0

  DaysInMonth           byte    31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31                        


dat

{{

  Copyright (C) 2014-2016 Jon McPhalen 

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}